@@ -9,9 +9,10 @@ from .base import unfold, tensor_to_vec
 from ._factorized_tensor import FactorizedTensor
 import warnings
 
+
 class Parafac2Tensor(FactorizedTensor):
-    """A wrapper class for the PARAFAC2 decomposition.
-    """
+    """A wrapper class for the PARAFAC2 decomposition."""
+
     def __init__(self, parafac2_tensor):
 
         super().__init__()
@@ -27,7 +28,7 @@ class Parafac2Tensor(FactorizedTensor):
         self.factors = factors
         self.weights = weights
         self.projections = projections
-        
+
     @classmethod
     def from_CPTensor(self, cp_tensor, parafac2_tensor_ok=False):
         """Create a Parafac2Tensor from a CPTensor
@@ -47,8 +48,10 @@ class Parafac2Tensor(FactorizedTensor):
         if parafac2_tensor_ok and len(cp_tensor) == 3:
             return Parafac2Tensor(cp_tensor)
         elif len(cp_tensor) == 3:
-            raise TypeError('Input is not a CPTensor. If it is a Parafac2Tensor, then the argument ``parafac2_tensor_ok`` must be True')
-        
+            raise TypeError(
+                "Input is not a CPTensor. If it is a Parafac2Tensor, then the argument ``parafac2_tensor_ok`` must be True"
+            )
+
         weights, (A, B, C) = cp_tensor
         Q, R = T.qr(B)
         projections = [Q for _ in range(T.shape(A)[0])]
@@ -60,23 +63,25 @@ class Parafac2Tensor(FactorizedTensor):
             return self.weights
         elif index == 1:
             return self.factors
-        elif index ==2:
+        elif index == 2:
             return self.projections
-        else: 
-            raise IndexError('You tried to access index {} of a PARAFAC2 tensor.\n'
-                             'You can only access index 0, 1 and 2 of a PARAFAC2 tensor'
-                             '(corresponding respectively to the weights, factors and projections)'.format(index))
-    
+        else:
+            raise IndexError(
+                f"You tried to access index {index} of a PARAFAC2 tensor.\n"
+                "You can only access index 0, 1 and 2 of a PARAFAC2 tensor"
+                "(corresponding respectively to the weights, factors and projections)"
+            )
+
     def __iter__(self):
         yield self.weights
         yield self.factors
         yield self.projections
-        
+
     def __len__(self):
         return 3
-    
+
     def __repr__(self):
-        message = '(weights, factors, projections) : rank-{} Parafac2Tensor of shape {} '.format(self.rank, self.shape)
+        message = f"(weights, factors, projections) : rank-{self.rank} Parafac2Tensor of shape {self.shape} "
         return message
 
     def to_tensor(self):
@@ -84,20 +89,20 @@ class Parafac2Tensor(FactorizedTensor):
 
     def to_vec(self):
         return parafac2_to_vec(self)
-    
+
     def to_unfolded(self, mode):
         return parafac2_to_unfolded(self, mode)
 
 
 def _validate_parafac2_tensor(parafac2_tensor):
     """Validates a parafac2_tensor in the form (weights, factors)
-    
+
         Returns the rank and shape of the validated tensor
-    
+
     Parameters
     ----------
     parafac2_tensor : Parafac2Tensor or (weights, factors)
-    
+
     Returns
     -------
     (shape, rank) : (int tuple, int)
@@ -108,15 +113,19 @@ def _validate_parafac2_tensor(parafac2_tensor):
         return parafac2_tensor.shape, parafac2_tensor.rank
 
     weights, factors, projections = parafac2_tensor
-            
+
     if len(factors) != 3:
-        raise ValueError('A PARAFAC2 tensor should be composed of exactly three factors.'
-                         'However, {} factors was given.'.format(len(factors)))
+        raise ValueError(
+            "A PARAFAC2 tensor should be composed of exactly three factors."
+            f"However, {len(factors)} factors was given."
+        )
 
     if len(projections) != factors[0].shape[0]:
-        raise ValueError('A PARAFAC2 tensor should have one projection matrix for each horisontal'
-                         ' slice. However, {} projection matrices was given and the first mode has'
-                         'length {}'.format(len(projections), factors[0].shape[0]))
+        raise ValueError(
+            "A PARAFAC2 tensor should have one projection matrix for each horisontal"
+            f" slice. However, {len(projections)} projection matrices was given and the first mode has"
+            f"length {factors[0].shape[0]}"
+        )
 
     rank = int(T.shape(factors[0])[1])
 
@@ -125,36 +134,36 @@ def _validate_parafac2_tensor(parafac2_tensor):
         current_mode_size, current_rank = T.shape(projection)
         if current_rank != rank:
             raise ValueError(
-                'All the projection matrices of a PARAFAC2 tensor should have the same number of '
-                'columns as the rank. However, rank={} but projections[{}].shape[1]={}'.format(
-                    rank, i, T.shape(projection)[1]
-                )
+                "All the projection matrices of a PARAFAC2 tensor should have the same number of "
+                f"columns as the rank. However, rank={rank} but projections[{i}].shape[1]={T.shape(projection)[1]}"
             )
 
         inner_product = T.dot(T.transpose(projection), projection)
         if T.max(T.abs(inner_product - T.eye(rank, **T.context(inner_product)))) > 1e-5:
             raise ValueError(
-                'All the projection matrices must be orthonormal, that is, P.T@P = I. '
-                'However, T.norm(projection[{}].T@projection[{}] - T.eye(rank)) = {}'.format(
-                    i, i, T.norm(inner_product - T.eye(rank, **T.context(inner_product))) 
-                )
+                "All the projection matrices must be orthonormal, that is, P.T@P = I. "
+                f"However, T.norm(projection[{i}].T@projection[{i}] - T.eye(rank)) = "
+                f"{T.norm(inner_product - T.eye(rank, **T.context(inner_product)))}"
             )
-        
-        shape.append((current_mode_size, *[f.shape[0] for f in factors[2:]]))  # Tuple unpacking to possibly support higher order PARAFAC2 tensors in the future
 
-    
+        shape.append(
+            (current_mode_size, *[f.shape[0] for f in factors[2:]])
+        )  # Tuple unpacking to possibly support higher order PARAFAC2 tensors in the future
+
     # Skip first factor matrix since the rank is extracted from it.
     for i, factor in enumerate(factors[1:]):
         current_mode_size, current_rank = T.shape(factor)
         if current_rank != rank:
-            raise ValueError('All the factors of a PARAFAC2 tensor should have the same number of columns.'
-                             'However, factors[0].shape[1]={} but factors[{}].shape[1]={}.'.format(
-                                 rank, i, T.shape(factor)[1]))
+            raise ValueError(
+                "All the factors of a PARAFAC2 tensor should have the same number of columns."
+                f"However, factors[0].shape[1]={rank} but factors[{i}].shape[1]={current_rank}."
+            )
+
+    if weights is not None and T.shape(weights)[0] != rank:
+        raise ValueError(
+            f"Given factors for a rank-{rank} PARAFAC2 tensor but len(weights)={T.shape(weights)[0]}."
+        )
 
-    if weights is not None and T.shape(weights)[0]  != rank:
-        raise ValueError('Given factors for a rank-{} PARAFAC2 tensor but len(weights)={}.'.format(
-            rank, T.shape(weights)[0]))
-        
     return tuple(shape), rank
 
 
@@ -184,38 +193,40 @@ def parafac2_normalise(parafac2_tensor):
     # allocate variables for weights, and normalized factors
     _, rank = _validate_parafac2_tensor(parafac2_tensor)
     weights, factors, projections = parafac2_tensor
-    
+
     # if (not copy) and (weights is None):
     #     warnings.warn('Provided copy=False and weights=None: a new Parafac2Tensor'
     #                   'with new weights and factors normalised inplace will be returned.')
     #     weights = T.ones(rank, **T.context(factors[0]))
-    
+
     # The if test below was added to enable inplace edits
     # however, TensorFlow does not support inplace edits
     # so this is always set to True
-    if True:  
+    if True:
         factors = [T.copy(f) for f in factors]
         projections = [T.copy(p) for p in projections]
         if weights is not None:
             factors[0] = factors[0] * weights
         weights = T.ones(rank, **T.context(factors[0]))
-        
+
     for i, factor in enumerate(factors):
         scales = T.norm(factor, axis=0)
-        weights = weights*scales
-        scales_non_zero = T.where(scales==0, T.ones(T.shape(scales), **T.context(factors[0])), scales)
-        factors[i] = factor/scales_non_zero
-        
+        weights = weights * scales
+        scales_non_zero = T.where(
+            scales == 0, T.ones(T.shape(scales), **T.context(factors[0])), scales
+        )
+        factors[i] = factor / scales_non_zero
+
     return Parafac2Tensor((weights, factors, projections))
 
 
 def apply_parafac2_projections(parafac2_tensor):
     """Apply the projection matrices to the evolving factor.
-    
+
     Parameters
     ----------
     parafac2_tensor : Parafac2Tensor
-        
+
     Returns
     -------
     (weights, factors) : ndarray, tuple
@@ -232,7 +243,7 @@ def apply_parafac2_projections(parafac2_tensor):
     weights, factors, projections = parafac2_tensor
 
     evolving_factor = [T.dot(projection, factors[1]) for projection in projections]
-    
+
     return weights, (factors[0], evolving_factor, factors[2])
 
 
@@ -243,13 +254,13 @@ def parafac2_to_slice(parafac2_tensor, slice_idx, validate=True):
     :math:`X_i`, of :math:`X` is given by
 
     .. math::
-    
+
         X_i = B_i diag(a_i) C^T,
-    
+
     where :math:`diag(a_i)` is the diagonal matrix whose nonzero entries are equal to
-    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i` 
+    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i`
     is a :math:`J_i \times R` factor matrix such that the cross product matrix :math:`B_{i_1}^T B_{i_1}`
-    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix. 
+    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix.
     To compute this decomposition, we reformulate the expression for :math:`B_i` such that
 
     .. math::
@@ -265,7 +276,7 @@ def parafac2_to_slice(parafac2_tensor, slice_idx, validate=True):
     .. math::
 
         X_{ijk} = \sum_{r=1}^R A_{ir} B_{ijr} C_{kr},
-    
+
     with the same constraints hold for :math:`B_i` as above.
 
     Parameters
@@ -277,7 +288,7 @@ def parafac2_to_slice(parafac2_tensor, slice_idx, validate=True):
             Contains the matrices :math:`A`, :math:`B` and :math:`C` described above
         * projection_matrices : List of projection matrices used to create evolving
             factors.
-    
+
     Returns
     -------
     ndarray
@@ -290,12 +301,12 @@ def parafac2_to_slice(parafac2_tensor, slice_idx, validate=True):
     weights, (A, B, C), projections = parafac2_tensor
     a = A[slice_idx]
     if weights is not None:
-        a = a*weights
+        a = a * weights
 
     Ct = T.transpose(C)
 
     B_i = T.dot(projections[slice_idx], B)
-    return T.dot(B_i*a, Ct)
+    return T.dot(B_i * a, Ct)
 
 
 def parafac2_to_slices(parafac2_tensor, validate=True):
@@ -309,13 +320,13 @@ def parafac2_to_slices(parafac2_tensor, validate=True):
     :math:`X_i`, of :math:`X` is given by
 
     .. math::
-    
+
         X_i = B_i diag(a_i) C^T,
-    
+
     where :math:`diag(a_i)` is the diagonal matrix whose nonzero entries are equal to
-    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i` 
+    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i`
     is a :math:`J_i \times R` factor matrix such that the cross product matrix :math:`B_{i_1}^T B_{i_1}`
-    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix. 
+    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix.
     To compute this decomposition, we reformulate the expression for :math:`B_i` such that
 
     .. math::
@@ -331,7 +342,7 @@ def parafac2_to_slices(parafac2_tensor, validate=True):
     .. math::
 
         X_{ijk} = \sum_{r=1}^R A_{ir} B_{ijr} C_{kr},
-    
+
     with the same constraints hold for :math:`B_i` as above.
 
     Parameters
@@ -343,7 +354,7 @@ def parafac2_to_slices(parafac2_tensor, validate=True):
             Contains the matrices :math:`A`, :math:`B` and :math:`C` described above
         * projection_matrices : List of projection matrices used to create evolving
             factors.
-    
+
     Returns
     -------
     List[ndarray]
@@ -355,7 +366,7 @@ def parafac2_to_slices(parafac2_tensor, validate=True):
         _validate_parafac2_tensor(parafac2_tensor)
     weights, (A, B, C), projections = parafac2_tensor
     if weights is not None:
-        A = A*weights
+        A = A * weights
         weights = None
 
     decomposition = weights, (A, B, C), projections
@@ -370,13 +381,13 @@ def parafac2_to_tensor(parafac2_tensor):
     :math:`X_i`, of :math:`X` is given by
 
     .. math::
-    
+
         X_i = B_i diag(a_i) C^T,
-    
+
     where :math:`diag(a_i)` is the diagonal matrix whose nonzero entries are equal to
-    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i` 
+    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i`
     is a :math:`J_i \times R` factor matrix such that the cross product matrix :math:`B_{i_1}^T B_{i_1}`
-    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix. 
+    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix.
     To compute this decomposition, we reformulate the expression for :math:`B_i` such that
 
     .. math::
@@ -392,7 +403,7 @@ def parafac2_to_tensor(parafac2_tensor):
     .. math::
 
         X_{ijk} = \sum_{r=1}^R A_{ir} B_{ijr} C_{kr},
-    
+
     with the same constraints hold for :math:`B_i` as above.
 
     Parameters
@@ -404,7 +415,7 @@ def parafac2_to_tensor(parafac2_tensor):
             Contains the matrices :math:`A`, :math:`B` and :math:`C` described above
         * projection_matrices : List of projection matrices used to create evolving
             factors.
-    
+
     Returns
     -------
     ndarray
@@ -413,8 +424,8 @@ def parafac2_to_tensor(parafac2_tensor):
     _, (A, _, C), projections = parafac2_tensor
     slices = parafac2_to_slices(parafac2_tensor)
     lengths = [projection.shape[0] for projection in projections]
-    
-    tensor = T.zeros((A.shape[0], max(lengths), C.shape[0]),  **T.context(slices[0]))
+
+    tensor = T.zeros((A.shape[0], max(lengths), C.shape[0]), **T.context(slices[0]))
     for i, (slice_, length) in enumerate(zip(slices, lengths)):
         tensor = T.index_update(tensor, T.index[i, :length], slice_)
     return tensor
@@ -427,13 +438,13 @@ def parafac2_to_unfolded(parafac2_tensor, mode):
     :math:`X_i`, of :math:`X` is given by
 
     .. math::
-    
+
         X_i = B_i diag(a_i) C^T,
-    
+
     where :math:`diag(a_i)` is the diagonal matrix whose nonzero entries are equal to
-    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i` 
+    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i`
     is a :math:`J_i \times R` factor matrix such that the cross product matrix :math:`B_{i_1}^T B_{i_1}`
-    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix. 
+    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix.
     To compute this decomposition, we reformulate the expression for :math:`B_i` such that
 
     .. math::
@@ -449,7 +460,7 @@ def parafac2_to_unfolded(parafac2_tensor, mode):
     .. math::
 
         X_{ijk} = \sum_{r=1}^R A_{ir} B_{ijr} C_{kr},
-    
+
     with the same constraints hold for :math:`B_i` as above.
 
     Parameters
@@ -461,7 +472,7 @@ def parafac2_to_unfolded(parafac2_tensor, mode):
             Contains the matrices :math:`A`, :math:`B` and :math:`C` described above
         * projection_matrices : List of projection matrices used to create evolving
             factors.
-    
+
     Returns
     -------
     ndarray
@@ -477,13 +488,13 @@ def parafac2_to_vec(parafac2_tensor):
     :math:`X_i`, of :math:`X` is given by
 
     .. math::
-    
+
         X_i = B_i diag(a_i) C^T,
-    
+
     where :math:`diag(a_i)` is the diagonal matrix whose nonzero entries are equal to
-    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i` 
+    the :math:`i`-th row of the :math:`I \times R` factor matrix :math:`A`, :math:`B_i`
     is a :math:`J_i \times R` factor matrix such that the cross product matrix :math:`B_{i_1}^T B_{i_1}`
-    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix. 
+    is constant for all :math:`i`, and :math:`C` is a :math:`K \times R` factor matrix.
     To compute this decomposition, we reformulate the expression for :math:`B_i` such that
 
     .. math::
@@ -499,7 +510,7 @@ def parafac2_to_vec(parafac2_tensor):
     .. math::
 
         X_{ijk} = \sum_{r=1}^R A_{ir} B_{ijr} C_{kr},
-    
+
     with the same constraints hold for :math:`B_i` as above.
 
     Parameters
@@ -511,7 +522,7 @@ def parafac2_to_vec(parafac2_tensor):
             Contains the matrices :math:`A`, :math:`B` and :math:`C` described above
         * projection_matrices : List of projection matrices used to create evolving
             factors.
-    
+
     Returns
     -------
     ndarray
