@@ -10,7 +10,7 @@ from ..utils import DefineDeprecated
 # License: BSD 3 clause
 
 
-class CPRegressor():
+class CPRegressor:
     """CP tensor regression
 
         Learns a low rank CP tensor weight
@@ -30,7 +30,15 @@ class CPRegressor():
         level of verbosity
     """
 
-    def __init__(self, weight_rank, tol=10e-7, reg_W=1, n_iter_max=100, random_state=None, verbose=1):
+    def __init__(
+        self,
+        weight_rank,
+        tol=10e-7,
+        reg_W=1,
+        n_iter_max=100,
+        random_state=None,
+        verbose=1,
+    ):
         self.weight_rank = weight_rank
         self.tol = tol
         self.reg_W = reg_W
@@ -39,9 +47,15 @@ class CPRegressor():
         self.verbose = verbose
 
     def get_params(self, **kwargs):
-        """Returns a dictionary of parameters
-        """
-        params = ['weight_rank', 'tol', 'reg_W', 'n_iter_max', 'random_state', 'verbose']
+        """Returns a dictionary of parameters"""
+        params = [
+            "weight_rank",
+            "tol",
+            "reg_W",
+            "n_iter_max",
+            "random_state",
+            "verbose",
+        ]
         return {param_name: getattr(self, param_name) for param_name in params}
 
     def set_params(self, **parameters):
@@ -68,7 +82,9 @@ class CPRegressor():
 
         # Initialise randomly the weights
         W = []
-        for i in range(1, T.ndim(X)):  # The first dimension of X is the number of samples
+        for i in range(
+            1, T.ndim(X)
+        ):  # The first dimension of X is the number of samples
             W.append(T.tensor(rng.randn(X.shape[i], self.weight_rank), **T.context(X)))
 
         # Norm of the weight tensor at each iteration
@@ -80,11 +96,18 @@ class CPRegressor():
             # Optimise each factor of W
             for i in range(len(W)):
                 phi = T.reshape(
-                          T.dot(partial_unfold(X, i, skip_begin=1),
-                                khatri_rao(W, skip_matrix=i)),
-                      (X.shape[0], -1))
-                inv_term = T.dot(T.transpose(phi), phi) + self.reg_W*T.tensor(np.eye(phi.shape[1]), **T.context(X))
-                W[i] = T.reshape(T.solve(inv_term, T.dot(T.transpose(phi), y)), (X.shape[i + 1], self.weight_rank))
+                    T.dot(
+                        partial_unfold(X, i, skip_begin=1), khatri_rao(W, skip_matrix=i)
+                    ),
+                    (X.shape[0], -1),
+                )
+                inv_term = T.dot(T.transpose(phi), phi) + self.reg_W * T.tensor(
+                    np.eye(phi.shape[1]), **T.context(X)
+                )
+                W[i] = T.reshape(
+                    T.solve(inv_term, T.dot(T.transpose(phi), y)),
+                    (X.shape[i + 1], self.weight_rank),
+                )
 
             weight_tensor_ = cp_to_tensor((weights, W))
             norm_W.append(T.norm(weight_tensor_, 2))
@@ -93,9 +116,9 @@ class CPRegressor():
             if iteration > 1:
                 weight_evolution = abs(norm_W[-1] - norm_W[-2]) / norm_W[-1]
 
-                if (weight_evolution <= self.tol):
+                if weight_evolution <= self.tol:
                     if self.verbose:
-                        print('\nConverged in {} iterations'.format(iteration))
+                        print(f"\nConverged in {iteration} iterations")
                     break
 
         self.weight_tensor_ = weight_tensor_
@@ -117,4 +140,5 @@ class CPRegressor():
         """
         return T.dot(partial_tensor_to_vec(X), self.vec_W_)
 
-KruskalRegressor = DefineDeprecated('KruskalRegressor', CPRegressor)
\ No newline at end of file
+
+KruskalRegressor = DefineDeprecated("KruskalRegressor", CPRegressor)
