@@ -18,7 +18,9 @@ def test_CPRegressor():
 
     # Generate random samples
     rng = T.check_random_state(1234)
-    X = T.tensor(rng.normal(size=(1200, image_height, image_width, n_channels), loc=0, scale=1))
+    X = T.tensor(
+        rng.normal(size=(1200, image_height, image_width, n_channels), loc=0, scale=1)
+    )
     regression_weights = np.zeros((image_height, image_width, n_channels))
     regression_weights[2:-2, 2:-2, 0] = 1
     regression_weights[2:-2, 2:-2, 1] = 2
@@ -27,18 +29,26 @@ def test_CPRegressor():
 
     y = T.dot(partial_tensor_to_vec(X, skip_begin=1), tensor_to_vec(regression_weights))
     X_train = X[:1000, :, :]
-    X_test = X[1000:, : ,:]
+    X_test = X[1000:, :, :]
     y_train = y[:1000]
     y_test = y[1000:]
 
-    estimator = CPRegressor(weight_rank=4, tol=10e-8, reg_W=1, n_iter_max=200, verbose=True)
+    estimator = CPRegressor(
+        weight_rank=4, tol=10e-8, reg_W=1, n_iter_max=200, verbose=True
+    )
     estimator.fit(X_train, y_train)
     y_pred = estimator.predict(X_test)
     error = RMSE(y_test, y_pred)
-    assert_(error <= tol, msg='CP Regressor : RMSE is too large, {} > {}'.format(error, tol))
+    assert_(error <= tol, msg=f"CP Regressor : RMSE is too large, {error} > {tol}")
 
     params = estimator.get_params()
-    assert_(params['weight_rank'] == 4, msg='get_params did not return the correct parameters')
-    params['weight_rank'] = 5
+    assert_(
+        params["weight_rank"] == 4,
+        msg="get_params did not return the correct parameters",
+    )
+    params["weight_rank"] = 5
     estimator.set_params(**params)
-    assert_(estimator.weight_rank == 5, msg='set_params did not correctly set the given parameters')
+    assert_(
+        estimator.weight_rank == 5,
+        msg="set_params did not correctly set the given parameters",
+    )
