@@ -3,8 +3,12 @@ import itertools
 import numpy as np
 
 from ... import backend as T
-from ...testing import (assert_array_equal, assert_equal,
-                        assert_array_almost_equal, assert_raises)
+from ...testing import (
+    assert_array_equal,
+    assert_equal,
+    assert_array_almost_equal,
+    assert_raises,
+)
 from ...base import fold, unfold
 
 
@@ -14,31 +18,21 @@ from .. import mode_dot, multi_mode_dot
 
 def test_mode_dot():
     """Test for mode_dot (n_mode_product)"""
-    X = T.tensor([[[1, 13],
-                   [4, 16],
-                   [7, 19],
-                   [10, 22]],
-
-                  [[2, 14],
-                   [5, 17],
-                   [8, 20],
-                   [11, 23]],
-
-                  [[3, 15],
-                   [6, 18],
-                   [9, 21],
-                   [12, 24]]])
+    X = T.tensor(
+        [
+            [[1, 13], [4, 16], [7, 19], [10, 22]],
+            [[2, 14], [5, 17], [8, 20], [11, 23]],
+            [[3, 15], [6, 18], [9, 21], [12, 24]],
+        ]
+    )
     # tensor times matrix
-    U = T.tensor([[1, 3, 5],
-                  [2, 4, 6]])
-    true_res = T.tensor([[[22, 130],
-                          [49, 157],
-                          [76, 184],
-                          [103, 211]],
-                         [[28, 172],
-                          [64, 208],
-                          [100, 244],
-                          [136, 280]]])
+    U = T.tensor([[1, 3, 5], [2, 4, 6]])
+    true_res = T.tensor(
+        [
+            [[22, 130], [49, 157], [76, 184], [103, 211]],
+            [[28, 172], [64, 208], [100, 244], [136, 280]],
+        ]
+    )
     res = mode_dot(X, U, 0)
     assert_array_equal(true_res, res)
 
@@ -47,17 +41,13 @@ def test_mode_dot():
     #######################
     # Test with a matrix
     U = T.tensor([1, 2, 3, 4])
-    true_res = T.tensor([[70, 190],
-                         [80, 200],
-                         [90, 210]])
+    true_res = T.tensor([[70, 190], [80, 200], [90, 210]])
     res = mode_dot(X, U, 1)
     assert_array_equal(true_res, res)
     # Test with a third order tensor
     X = T.tensor(np.arange(24, dtype=float).reshape((3, 4, 2)))
     v = T.tensor(np.arange(4, dtype=float))
-    true_res = ([[ 28,  34],
-                 [ 76,  82],
-                 [124, 130]])
+    true_res = [[28, 34], [76, 82], [124, 130]]
     res = mode_dot(X, v, 1)
     assert_array_equal(true_res, res)
 
@@ -66,7 +56,7 @@ def test_mode_dot():
     U = T.tensor(np.random.random((3, 4)))
     true_res = fold(T.dot(U, unfold(X, 1)), 1, (2, 3, 5))
     res = mode_dot(X, U, 1)
-    assert (res.shape == (2, 3, 5))
+    assert res.shape == (2, 3, 5)
     assert_array_almost_equal(true_res, res)
 
     #########################################
@@ -100,10 +90,8 @@ def test_multi_mode_dot():
     * U_1 x unfold(X, 1) x kronecker(U_2, ..., U_n).T
     * U_1 x unfold(X x_2 U_2 x ... x_n U_n)
     """
-    X = T.tensor([[1, 2],
-                  [0, -1]])
-    U = [T.tensor([2, 1]),
-         T.tensor([-1, 1])]
+    X = T.tensor([[1, 2], [0, -1]])
+    U = [T.tensor([2, 1]), T.tensor([-1, 1])]
     true_res = T.tensor([1])
     res = multi_mode_dot(X, U, [0, 1])
     assert_array_equal(true_res, res)
@@ -117,17 +105,25 @@ def test_multi_mode_dot():
     dims = [2, 3, 4, 5]
     X = T.tensor(np.random.randn(*dims))
     factors = [T.tensor(np.random.rand(dims[i], X.shape[i])) for i in range(T.ndim(X))]
-    true_res = T.dot(T.dot(factors[0], unfold(X, 0)), T.transpose(kronecker(factors[1:])))
+    true_res = T.dot(
+        T.dot(factors[0], unfold(X, 0)), T.transpose(kronecker(factors[1:]))
+    )
     n_mode_res = multi_mode_dot(X, factors)
     assert_array_almost_equal(true_res, unfold(n_mode_res, 0), decimal=5)
     for i in range(T.ndim(X)):
         indices = [j for j in range(T.ndim(X)) if j != i]
         sub_factors = [factors[j] for j in indices]
-        true_res = T.dot(T.dot(factors[i], unfold(X, i)), T.transpose(kronecker(sub_factors)))
+        true_res = T.dot(
+            T.dot(factors[i], unfold(X, i)), T.transpose(kronecker(sub_factors))
+        )
         res = unfold(n_mode_res, i)
         temp = multi_mode_dot(X, sub_factors, modes=indices)
         res2 = T.dot(factors[i], unfold(temp, i))
-        assert_equal(true_res.shape, res.shape, err_msg='shape should be {}, is {}'.format(true_res.shape, res.shape))
+        assert_equal(
+            true_res.shape,
+            res.shape,
+            err_msg=f"shape should be {true_res.shape}, is {res.shape}",
+        )
         assert_array_almost_equal(true_res, res, decimal=5)
         assert_array_almost_equal(true_res, res2, decimal=5)
 
@@ -150,7 +146,7 @@ def test_multi_mode_dot():
 
     # Average pooling each mode
     # Order should not matter
-    vecs = [vecs[i]/s for i, s in enumerate(shape)]
+    vecs = [vecs[i] / s for i, s in enumerate(shape)]
     for modes in itertools.permutations(range(len(shape))):
         res = multi_mode_dot(X, [vecs[i] for i in modes], modes=modes)
         assert_equal(T.shape(res), ())
